        program MevasCertify
c
c       This program reads in data, calls pole placement subroutine,
c       computes eigenvalues of closed loop, and writes results.
c
c       .. Parameters ..
c       implicit none
        integer          Nin, Nout
        parameter        (Nin = 5, Nout = 6)
        integer          Nmax, Mmax
        parameter        (Nmax = 20, Mmax = 20)
        integer          lda, ldb
        parameter        (lda=Nmax, ldb=Nmax)
c       .... for upper bounds on Givens and Householder transformations
c            with N in {1,..,Nmax} and M in {1,..,min(N,Mmax)} 
c                the expressions for gmax and hmax yield
c                greatest gmax = 56 when N=20, M=10
c                greatest hmax = 101 when N=20, M=1
        integer          gmax
        parameter        (gmax = 56)
        integer          hmax
        parameter        (hmax = 101)
c       .... and for the work space bounds
        integer          liwork
        parameter        (liwork = Nmax + Nmax/2 + gmax + hmax)
        integer          lrwork
        parameter        (lrwork = 3*Nmax + 2*gmax + 3*hmax)
c
c       .. Local Scalars ..
        double precision tol
        integer          n, m, kmax, ncmplx, iwarn, ierr, i, j
        integer          m0, c1
        character*20     header
c
c       .. Local Arrays ..
        double precision A(lda,Nmax), B(ldb,Mmax), eigs(Nmax)
        double precision rwork(lrwork)
        integer          kstair(Nmax+1), info(2), iwork(liwork)
        double precision AA(lda,Nmax), BB(ldb,Mmax)
        double precision reigs(Nmax), imeigs(Nmax)
c
c       .. External Subroutines ..      
        external dmvsli, lpeigs
c
c       .. Executable Statements ..
c
c       .. read and echo the heading in the data file
        read (Nin,FMT=99990) header
        write (Nout,FMT=99999)
        write (Nout,FMT=99990) header
c
c       .. read the data ..
        read (Nin,FMT=*) n, m, tol
        if (n.le.0 .or. n.gt.Nmax) then
           write (Nout,FMT=99998) n
        else
           read (Nin,FMT=*) (( A(i,j), j=1,n), i=1,n)
           if (m.le.0 .or. m.gt.Mmax) then
              write (Nout,FMT=99997) m
           else
              read (Nin,FMT=*) (( B(i,j), j=1,m), i=1,n)
              read (Nin,FMT=*) ( eigs(i), i=1,n )
              read (Nin,FMT=*) ncmplx 
              read (Nin,FMT=*) kmax
              if (kmax.le.0 .or. kmax.gt.Nmax) then
                 write (Nout,FMT=99996) kmax
              else
                 read (Nin,FMT=*) (kstair(i), i=1,kmax)
c                .. make sure kstair(kmax+1) = 0 ..
                 kstair(kmax+1) = 0
c
c                .. copy A to AA ..
                 do 100 j = 1, n
                    call dcopy(n, A(1,j), 1, AA(1,j), 1)
  100            continue
c                .. copy B to BB ..
                 do 120 j = 1, m
                    call dcopy(n, B(1,j), 1, BB(1,j), 1)
  120            continue
c
c                .. echo the eigenvalues to be allocated
                 write(Nout,FMT=80058)
                 do 150 i = 1, ncmplx, 2
                    write(Nout,FMT=80054) EIGS(i),EIGS(i+1)           
                    write(Nout,FMT=80055) EIGS(i),EIGS(i+1)           
  150            continue
                 do 170 i = ncmplx+1, n
                    write(Nout,FMT=80056) EIGS(I)
  170            continue
c
c                .. allocate the eigenvalues ..
                 call dmvsli (n,m, ncmplx, gmax, hmax, 
     &                          A,lda, B,ldb, eigs, kstair,
     &                          info, iwork, rwork, tol, iwarn, ierr)
c
                 write (Nout,FMT='()')
                 if (ierr .lt. 0) then
                    write(Nout,FMT=80000) -ierr
                 else
c                   .. print results ..
                    if (iwarn .ne. 0) then
                       write (Nout,FMT=80020) iwarn
                    end if
                    if (ierr .ne. 0) then
                       write(Nout,FMT=80010) ierr
                    end if
                    write (Nout,FMT=80030) tol
                    if (info(2) .ne. n) then
                       write (Nout,FMT=80040)
                       write (Nout,FMT=80041) info(2)
                    end if
                    if (info(1) .ne. n) then
                       write (Nout,FMT=80050) n, info(1)
c                      .. print UNallocated eigenvalues ..
                       write (Nout,FMT=80052)
                       do 200 i=info(1)+1,info(1)+ncmplx,2
                          write(Nout,FMT=80054) EIGS(i),EIGS(i+1)           
                          write(Nout,FMT=80055) EIGS(i),EIGS(i+1)           
  200                  continue
                       do 220 i = info(1)+1+ncmplx, n
                          write(Nout,FMT=80056) EIGS(I)
  220                  continue
                    end if
c
c                   .. before printing F compute and print eigenvalues
c                         of the closed loop ..
c                   .. compute the closed loop and its eigenvalues ..
c                   ! lpeigs will overwrite AA
                    if ( m .GT. n) then
                        m0 = n
                        c1 = m-n+1
                    else
                        m0 = m
                        c1 = 1
                    endif
                    call lpeigs(n,m0, AA,lda, BB(1,c1), ldb, A,lda,
     &                             reigs, imeigs, iwork, rwork)
c
c                   .. print computed eigenvalues of closed loop ..
c                   .. imaginary parts with magnitude < tol are set to zero ..
                    write (Nout,FMT=80060)
                    DO 400 i=1,n
                        if ( abs(imeigs(i)) .LE. tol ) then
                           write(Nout,FMT=80056) reigs(i)
                        else if ( imeigs(i) .GE. 0.0 ) then
                           write(Nout,FMT=80054) reigs(i),imeigs(i)           
                        else
                           write(Nout,FMT=80055) reigs(i),-imeigs(i)
                        endif
  400               continue
c
c                   .. print computed F ..
                    write (Nout,FMT='()')
                    write (Nout,FMT=80080)
                    DO 500 i = 1, min(m,n)
                        write(Nout,FMT=88888) (A(i,j), j=1,n)
  500               continue
c
                 end if
              end if
           end if
        end if
c
80000   FORMAT (' ERROR: error on ENTRY with argument ', I2)
80010   FORMAT (' ERROR: on EXIT ierr  = ', I2)
80020   FORMAT (' WARNING: on exit iwarn = ', I1)
80030   FORMAT (' tolerance used = ', E16.8)
80040   FORMAT (' eigenvalue stored at EIGS(N) on entry was allocated')
80041   FORMAT ('   out of sequence:  now stored at EIGS(', I2, ')')
80050   FORMAT (' of', I3, ' eigenvalues, the number allocated = ', I2)
80052   FORMAT (' the following eigenvalues were NOT allocated')
80054   FORMAT (F8.4, '  +  i*', F8.4)
80055   FORMAT (F8.4, '  -  i*', F8.4)
80056   FORMAT (F8.4)
80058   FORMAT (' the eigenvalues to be allocated are:')
80060   FORMAT (' the eigenvalues of the closed loop are:')
80080   FORMAT (' computed gain matrix F:')
88888   FORMAT (20(1x,F9.4))
99990   FORMAT (A20)
99996   FORMAT (' kmax is out of range: kmax = ', I2)
99997   FORMAT (' m is out of range: m = ', I2)
99998   FORMAT (' n is out of range: n = ', I2)
99999   FORMAT (' Certification Program Results')
c
        stop
        end
c===================================================================
c
        subroutine lpeigs(n, m, A,lda, B,ldb, F,ldf, reig, imeig,
     &                    iwork, rwork)
c
c    Purpose
c    =======
c    To call routines to compute A-BF and the eigenvalues of A-BF.
c
c    Arguments
c    =========

c    Arguments In
c    ------------
c    N      INTEGER.
c           Row and column dimension of matrix A,
c           row dimension of matrix B,
c           column dimension of matrix F.
c
c    M      INTEGER.
c           Column dimension of matrix B,
c           row dimension of matrix F.
c
c    A      DOUBLE PRECISION array of DIMENSION (LDA,N).
c           The leading N by N part of this array must contain the matrix A.
c           Note: this array is overwritten.
c
c    LDA    INTEGER.
c           Row dimension of array A, as declared in the calling program
c           LDA .ge. N
c
c    B      DOUBLE PRECISION array of DIMENSION (LDB,M).
c           The leading N by M part of this array must contain the matrix B.
c
c    LDB    INTEGER.
c           Row dimension of array B, as declared in the calling program
c           LDB .ge. N.
c
c    F      DOUBLE PRECISION array of DIMENSION (LDF,N).
c           The leading M by N part of this array must contain the matrix F.
c
c    LDF    INTEGER.
c           Row dimension of array F, as declared in the calling program
c           LDB .ge. M.
c
c    Arguments Out
c    -------------
c    A      DOUBLE PRECISION array of DIMENSION (LDA,N).
c           The leading M by N part of this array contains the computed
c           gain matrix "F".
c
c    REIG   DOUBLE PRECISION array of DIMENSION(N).
c           Contains the real parts of the computed eigenvalues.
c
c    IMEIG  DOUBLE PRECISION array of DIMENSION(N).
c           Contains the imaginary parts of the computed eigenvalues.
c
c    Workspace
c    ---------
c    IWORK  INTEGER array of DIMENSION(N).
c
c    RWORK  DOUBLE PRECISION array of DIMENSION(N).
c
c    Tolerances
c    ----------
c    None.
c
c    Mode Parameters
c    ---------------
c    None.
c    
c    Warning Indicator
c    -----------------
c    None.
c
c    Error Indicator
c    ---------------
c    None.
c
c    Warnings and Errors Detected by the Routine
c    ===========================================
c    None
c
c    Method
c    ======
c    Uses BLAS routine DGEMM to compute B-AF.
c    Subsequent calls to EISPACK routines BALANC, ELMHES, HQR 
c    balance the matrix, reduce it to upper hessenberg form, and
c    compute the eigenvalues via the QR algorithm.
c
c    References
c    ==========
c    1. Golub, G.H. and Van Loan, C.F., Matrix Computations, 2-nd ed.,
c       Johns Hopkins University Press, Baltimore, 1989, Chapter 7.  
c    
c    2. Press, W.H. et al, Numerical Recipes, Cambridge University Press,
c       1986, pp.365-376
c
c    Contributors
c    ============
c    H. Roth  (Memorial University of Newfoundland, Canada)
c
c
c    Revisions
c    =========
c    1992 Oct 20
c
c    arguments
c       implicit none
        integer n, m, lda, ldb, ldf, iwork(*)
        double precision A(lda,*), B(ldb,*), F(ldf,*)
        double precision reig(*), imeig(*), rwork(*)
c
c    parameters
        character*1 Tran
        parameter(Tran='n')
c
c    local variables
        integer low,igh,ierr
c
c       ..compute closed loop A-B*F and store in A
        call dgemm(Tran,Tran, n,n,m, -1.0d0, B,ldb, F,ldf, 1.0d0, A,lda)
c
c       ..compute eigenvalues of the closed loop ( stored in A )
        call balanc( lda, n, A, low, igh, rwork)
        call elmhes( lda, n, low, igh, A, iwork)
        call hqr( lda, n, low, igh, A, reig, imeig, ierr)
c
        return
        end

