        program MevasExample
c
c       This program reads in data, calls pole placement subroutine,
c       and writes results.
c
c       .. Parameters ..
c       implicit none
        integer          Nin, Nout
        parameter        (Nin = 5, Nout = 6)
        integer          Nmax, Mmax
        parameter        (Nmax = 20, Mmax = 20)
        integer          lda, ldb
        parameter        (lda=Nmax, ldb=Nmax)
c       .... for number of Givens and Householder transformations
c            with N in {1,..,Nmax} and M in {1,..,min(N,Mmax)} 
c                the expressions for gmax and hmax yield
c                greatest gmax = 56 when N=20, M=10
c                greatest hmax = 101 when N=20, M=1
        integer          gmax
        parameter        (gmax = 56)
        integer          hmax
        parameter        (hmax = 101)
c       .... and for the work space bounds
        integer          liwork
        parameter        (liwork = Nmax + Nmax/2 + gmax + hmax)
        integer          lrwork
        parameter        (lrwork = 3*Nmax + 2*gmax + 3*hmax)
c       .. Local Scalars ..
        double precision tol
        integer          n, m, kmax, ncmplx, iwarn, ierr, i, j
c       .. Local Arrays ..
        double precision A(lda,Nmax), B(ldb,Mmax), eigs(Nmax)
        double precision Rwork(lrwork)
        integer          kstair(Nmax+1), info(2), Iwork(liwork)
c       .. External Subroutines ..      
        external dmvsli
c       .. Executable Statements ..
c
        write (Nout, FMT=99999)
c
c       .. skip the heading in the data file and read the data ..
        read (Nin, FMT='()')
        read (NIN,FMT=*) n, m, tol
        if (n.le.0 .or. n.gt.Nmax) then
           write (Nout,FMT=99998) n
        else
           read (Nin,FMT=*) (( A(i,j), j=1,n), i=1,n)
           if (m.le.0 .or. m.gt.Mmax) then
              write (Nout,FMT=99997) m
           else
              read (Nin,FMT=*) (( B(i,j), j=1,m), i=1,n)
              read (Nin,FMT=*) ( eigs(i), i=1,n )
              read (Nin,FMT=*) ncmplx 
              read (Nin,FMT=*) kmax
              if (kmax.le.0 .or. kmax.gt.Nmax) then
                 write (Nout,FMT=99996) kmax
              else
                 read (Nin,FMT=*) (kstair(i), i=1,kmax)
c                .. make sure kstair(kmax+1) = 0 ..
                 kstair(kmax+1) = 0
c                .. allocate the eigenvalues ..
                 call DMVSLI(n,m, ncmplx, gmax, hmax, 
     &                          A,lda, B,ldb, eigs, kstair,
     &                          info, iwork, rwork, tol, iwarn, ierr)
c
                 write (Nout,FMT='()')
                 if (ierr .lt. 0) then
                    write(Nout,FMT=80000) -ierr
                 else
c                   .. print results ..
                    if (iwarn .ne. 0) then
                       write (Nout,FMT=80020) iwarn
                    end if
                    if (ierr .ne. 0) then
                       write(Nout,FMT=80010) ierr
                    end if
                    write (Nout,FMT=80030) tol
                    if (info(2) .ne. n) then
                       write (Nout,FMT=80040)
                       write (Nout,FMT=80041) info(2)
                    end if
                    if (info(1) .ne. n) then
                       write (Nout,FMT=80050) n, info(1)
c                      .. print UNallocated eigenvalues ..
                       write (Nout,FMT=80052)
                       do 200 i=info(1)+1,info(1)+ncmplx,2
                          write(NOUT,FMT=80054) EIGS(i),EIGS(i+1)           
                          write(NOUT,FMT=80055) EIGS(i),EIGS(i+1)           
  200                  continue
                       do 220 i = info(1)+1+ncmplx, n
                          write(NOUT,FMT=80056) EIGS(I)
  220                  continue
                    end if
c                   .. print computed F ..
                    write (Nout,FMT='()')
                    write (NOUT,FMT=80080)
                    DO 300 i = 1, min(m,n)
                        write(Nout,FMT=88888) (A(i,j), j=1,n)
  300               continue
                 end if
              end if
           end if
        end if
c
80000   FORMAT (' ERROR: error on ENTRY with argument ', I2)
80010   FORMAT (' ERROR: on EXIT ierr  = ', I2)
80020   FORMAT (' WARNING: on exit iwarn = ', I1)
80030   FORMAT (' tolerance used = ', E16.8)
80040   FORMAT (' eigenvalue stored at EIGS(N) on entry was allocated')
80041   FORMAT ('   out of sequence:  now stored at EIGS(', I2, ')')
80050   FORMAT (' of', I3, ' eigenvalues, the number allocated = ', I2)
80052   FORMAT (' the following eigenvalues were NOT allocated')
80054   FORMAT (F8.4, '  +  i*', F8.4)
80055   FORMAT (F8.4, '  -  i*', F8.4)
80056   FORMAT (F8.4)
80080   FORMAT (' computed gain matrix F:')
88888   FORMAT (20(1x,F9.4))
99996   FORMAT (' kmax is out of range: kmax = ', I2)
99997   FORMAT (' m is out of range: m = ', I2)
99998   FORMAT (' n is out of range: n = ', I2)
99999   FORMAT (' Example Program Results')
c
        stop
        end
