CXXXXXXXXXXXXXXXXXXXXX START OF FILE: DFLRD.F XXXXXXXXXXXXXXXXXXXXX

C*********************************************************************
C*********************************************************************
C
C                    DRIVER PROGRAM FOR  DSEVAS
C
C       This program prompts the user for a file name that
C       contains the input, reads the file, allocates the desired
C       eigenvalues, and finds the eigenvalues of the 
C       matrix (A - b*f).  The input values, output vector f,
C       and the resulting eigenvalues are printed.
C
C       INPUT FORMAT:
C       -------------
C
C       1) Line 1 of the file contains an integer in free format
C               showing the size of the system.
C
C       2) Line 2 of the file contains a logical value in free format
C               showing the form of the system.  A value of  "T"
C               indicates a system in controllability form, whereas
C               a value of  "F"  indicates a system in general form.
C
C       3) Line 3 of the file contains a real value in free format
C               giving the tolerance used for testing for
C               controllability. If the value  0.0  is given
C               the subroutine calculates the tolerance.
C
C       4) The matrix A is read next, one element per line,
C               in free format.  The matrix is read by row.
C
C       5) The vector b is read next, one element per line,
C               in free format.
C
C       6) The desired eigenvalues are read next.  Each line
C               contains an ordered pair of two real, free format 
C               values which represent the real and the 
C               imaginary part of a complex eigenvalue.
C               Real eigenvalues should obviously be represented
C               by an ordered pair with a zero imaginary part.
C               Complex conjugates should appear both, and 
C               contiguously. Real eigenvalues should appear in 
C               groups of even numbers, unless they are at the end 
C               of the list. A good way to list the eigenvalues 
C               would be, to first list all the complex eigenvalues 
C               followed be the real eigenvalues.
C
C   A typical example could be the following:
C
C   4
C   T
C   0.0
C   1    For      A= 1 2 3 4
C   2                5 6 7 8
C   3                0 9 1 2
C   4                0 0 3 4
C   5             
C   6
C   7
C   8
C   0
C   9
C   1
C   2
C   0
C   0
C   3
C   4
C   10   For      b= 10
C   0                 0
C   0                 0
C   0                 0
C   1  2
C   1 -2
C   3  0
C   4  0
C       
C       Note that in this test driver program we have set a limit
C       of  25  for the size of the matrix  A. This obviously
C       can be altered by the user.
C
C*********************************************************************
C*********************************************************************


        PROGRAM SAMPLE

        PARAMETER (ISL = 25, JSL = 2*ISL*ISL)
        DOUBLE PRECISION A(ISL,ISL),AA(ISL,ISL),B(ISL),BB(ISL)
        DOUBLE PRECISION F(ISL),RST(JSL)
        INTEGER IST(JSL),N,IEAL,IWRKDX,IWARN,IERR
        COMPLEX*16 EIG(ISL)
        DOUBLE PRECISION L(2,ISL),TOL
        LOGICAL FORM
        CHARACTER*20 FNAME


C
C.......read matrix A, vector b, and eigenvalue matrix L
C
        WRITE(*,*)
        WRITE(*,*) '*****************************'
        WRITE(*,*) '     DSEVAS TEST PROGRAM'
        WRITE(*,*) '*****************************'
        WRITE(*,*)
        WRITE(*,4000) 
4000    FORMAT(1X,'DATA FILE: ',/)
        READ(*,7000) FNAME
7000    FORMAT(A)
        OPEN(5, FILE=FNAME, STATUS='OLD')
        READ(5,*) N

        IF(N .GT. ISL) THEN
                WRITE(*,*) 'INPUT SYSTEM TOO LARGE.'
                STOP
        ENDIF
        READ(5,*) FORM
        READ(5,*) TOL
        DO 100 I=1,N    
            DO 100 J=1,N
                        READ(5,*) A(I,J) 
                        AA(I,J) = A(I,J)
100     CONTINUE

        DO 110 I=1,N
                READ(5,*) B(I)
                BB(I) = B(I)
110     CONTINUE

        DO 120 I=1,N
                READ(5,*) L(1,I), L(2,I)
120     CONTINUE

        CLOSE(5)

C
C.......Echo input
C
        WRITE(*,*)
        WRITE(*,*)
        WRITE(*,*) 'SYSTEM MATRIX (b,A):'
        WRITE(*,*)
        DO 130 I=1,N
                WRITE(*,5000) B(I),(A(I,J),J=1,N)
130     CONTINUE

        WRITE(*,*)
        WRITE(*,*) 'EIGENVALUES TO BE ALLOCATED:'
        WRITE(*,*)
        DO 135 I=1,N
                WRITE(*,5000) L(1,I),L(2,I)
135     CONTINUE

C
C.......call DSEVAS routine
C

        CALL DSEVAS(N,A,ISL,B,L,IEAL,IWRKDX,F,IST,RST,TOL,FORM,
     +                    IWARN,IERR)

       IF(IERR .ne. 0) THEN
            WRITE(*,5003)
            STOP
       ENDIF
       IF(IWARN .eq. 1) THEN
            WRITE(*,5010)
            WRITE(*,5005) IEAL
       ELSEIF(IWARN .eq. 2) THEN
            WRITE(*,5001)
            WRITE(*,5005) IEAL
       ENDIF

C
C.......create (A - b*f)
C
        WRITE(*,*)
        WRITE(*,*) 'RESULT VECTOR F:'
        WRITE(*,*)
        DO 140 I=1,N
                WRITE(*,*) F(I)
                AA(1,I) = AA(1,I) - BB(1)*F(I)
140     CONTINUE




C
C.......find eigenvalues of (A - b*f)
C
        CALL DEIGVL(N,AA,ISL,EIG)

        WRITE(*,*)
        WRITE(*,*) 'EIGENVALUES OF (A - B*F): '
        WRITE(*,*)
        DO 150 I=1,N
                WRITE(*,*) EIG(I)
150     CONTINUE



5000    FORMAT(1X,10E15.6)

5001    FORMAT(1X,'SYSTEM IS CLOSE TO UNCONTROLLABLE')
5002    FORMAT(1X,'USING TOLERANCE ',E20.12)
5003    FORMAT(1X,'FATAL INPUT ERROR')
5005    FORMAT(1X,I4,' EIGENVALUES ALLOCATED.')
5010    FORMAT(1X,'COMPLEX EIGENVALUES ARE NOT COMPLEX CONJUGATES.')

        STOP
        END

